/*********************************************************************
** AreaBuilder is a program designed to build maps for MUDS like    **
** Anatolia.                                                        **
** Copyright (C) 2003 Pierre Chatelier                              **
**                                                                  **
** This program is free software; you can redistribute              **
** it and/or modify it under the terms of the GNU                   **
** General Public License as published by the                       **
** Free Software Foundation, version 2.                             **
**                                                                  **
** This program is distributed in the hope that it will be          **
** useful, but WITHOUT ANY WARRANTY; without even the implied       **
** warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. **
** See the GNU General Public License for more details.             **
**                                                                  **
** You should have received a copy of the                           **
** GNU General Public License along with this program; if not,      **
** write to the Free Software Foundation,                           **
** Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA      **
*********************************************************************/
package areabuilder;

import java.awt.*;
import java.util.*;

public abstract class AbstractSelection
{
  protected LinkedList listOfSelectedObjects = null;

  public AbstractSelection()
  {
      listOfSelectedObjects = new LinkedList();
  }

  public int getSize()
  {
     return  listOfSelectedObjects.size();
  }

  protected void copy(AbstractSelection source)
  {
     this.listOfSelectedObjects = new LinkedList(source.listOfSelectedObjects);
     Iterator it = source.listOfSelectedObjects.listIterator();
     while (it.hasNext()) ((SelectableNodeObject) it.next()).setSelected(true);
  }


  public void clear()
  {
     clearSelection();
  }

  public boolean isEmpty() {return listOfSelectedObjects.isEmpty();}
  public boolean contains(SelectableNodeObject o) {return listOfSelectedObjects.contains(o);}

  public void clearSelection()
  {
     Iterator it = listOfSelectedObjects.iterator();
     while (it.hasNext())
     {
        ((SelectableNodeObject) it.next()).setSelected(false);
        it.remove();
     }
  }

  public void resetSelection() {clearSelection();}
  public void resetSelection(SelectableNodeObject o)
  {
    clearSelection();
    addToSelection(o);
  };

  public void toggleInSelection(SelectableNodeObject o) //add or remove according to previous state
  {
     if (o.isSelected()) removeFromSelection(o);
     else addToSelection(o);
  }

  public void addToSelection(SelectableNodeObject o)
  {
      listOfSelectedObjects.add(o);
      o.setSelected(true);
  }
  public void removeFromSelection(SelectableNodeObject o)
  {
     listOfSelectedObjects.remove(o);
     o.setSelected(false);
  }

}