/*********************************************************************
** AreaBuilder is a program designed to build maps for MUDS like    **
** Anatolia.                                                        **
** Copyright (C) 2003 Pierre Chatelier                              **
**                                                                  **
** This program is free software; you can redistribute              **
** it and/or modify it under the terms of the GNU                   **
** General Public License as published by the                       **
** Free Software Foundation, version 2.                             **
**                                                                  **
** This program is distributed in the hope that it will be          **
** useful, but WITHOUT ANY WARRANTY; without even the implied       **
** warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. **
** See the GNU General Public License for more details.             **
**                                                                  **
** You should have received a copy of the                           **
** GNU General Public License along with this program; if not,      **
** write to the Free Software Foundation,                           **
** Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA      **
*********************************************************************/
package areabuilder;

import javax.swing.tree.*;
import java.util.*; //for HashMap and LinkedList
import java.io.*;   //for archivation

public class AnatoliaObject extends AnatoliaThing
{
   //values...
   String name = "an object";
   String shortDescription = "this object is boring";
   String longDescription = "this object is very very boring";
   LinkedList extraDescriptions = new LinkedList() ; //Pair<keyword, description>
   String material = AnatoliaConsts.objectMaterialTexts[0];
   int type = AnatoliaConsts.OBJECT_TYPE_WEAPON;
   String extraFlags = "";
   String wearFlags = "";
   String v0 = "0";
   String v1 = "0";
   String v2 = "0";
   String v3 = "0";
   String v4 = "0";
   byte level = AnatoliaConsts.LEVEL_MIN;
   int weight = 1;
   int cost = 1;
   String cond = "P";
   String immFlags = "";
   String resFlags = "";
   String vulnFlags = "";
   String detectFlags = "";
   String generalAffects = "";
   int[] localAffects = new int[AnatoliaConsts.applyTexts.length];
   String[] specialProgs = new String[AnatoliaConsts.objectProgTexts.length]; //list of String for special prog
   int limit = 0;

   //instances...
   LinkedList instances = new LinkedList();//InstanceOfObject list

   public AnatoliaObject(AnatoliaCollection anatoliaCollection)
   {
      super(anatoliaCollection);
      for(int i=0 ; i<specialProgs.length ; ++i)
        specialProgs[i] = "";
   }

  public String toString()
  {
     return "obj<"+name+">;vnum "+super.getVnum();
  }

  public void addInstance(InstanceOfObject instance)
  {
     instances.add(instance);
  }
  public void removeInstance(InstanceOfObject instance)
  {
     instances.remove(instance);
  }


  public void clear()
  {
     extraDescriptions.clear();
     instances.clear();
     for(int i=0 ; i<specialProgs.length ; ++i)
        specialProgs[i] = "";
  }

  public void writeObject(ObjectOutputStream output) throws IOException
  {
     super.writeObject(output);

     //instances...
     Iterator it = instances.iterator();
     output.writeInt(instances.size());
     while(it.hasNext())
     {
        output.writeLong(((InstanceOfObject) it.next()).getObjectID());
     }

     output.writeInt(super.getVnum());
     output.writeUTF(name);
     output.writeUTF(shortDescription);
     output.writeUTF(longDescription);
     it = extraDescriptions.iterator();
     Pair pair = null;
     output.writeInt(extraDescriptions.size());
     while(it.hasNext())
     {
        pair = (Pair) it.next();
        output.writeUTF((String) pair.getFirst());
        output.writeUTF((String) pair.getSecond());
     }
     output.writeUTF(material);
     output.writeInt(type);
     output.writeUTF(extraFlags);
     output.writeUTF(wearFlags);
     output.writeUTF(v0);
     output.writeUTF(v1);
     output.writeUTF(v2);
     output.writeUTF(v3);
     output.writeUTF(v4);
     output.writeByte(level);
     output.writeInt(weight);
     output.writeInt(cost);
     output.writeUTF(cond);
     output.writeUTF(immFlags);
     output.writeUTF(resFlags);
     output.writeUTF(vulnFlags);
     output.writeUTF(detectFlags);
     output.writeUTF(generalAffects);

     output.writeInt(localAffects.length);
     for(int i=0 ; i<localAffects.length ; ++i)
     {
        output.writeInt(localAffects[i]);
     }

     output.writeInt(specialProgs.length);
     for(int i=0 ; i<specialProgs.length ; ++i)
       output.writeUTF(specialProgs[i]);

     output.writeInt(limit);
 }

  public void readObject(ObjectInputStream input) throws IOException, ClassNotFoundException
  {
     super.readObject(input);

     clear();

     //instances...
     int tempSize = input.readInt();
     Long instance = null;
     for(int i=0 ; i<tempSize ; ++i)
     {
        instance = new Long(input.readLong());
        instances.add(instance);
     }

     setVnum(input.readInt());
     name = input.readUTF();
     shortDescription = input.readUTF();
     longDescription = input.readUTF();

     tempSize = input.readInt();
     for(int i=0 ; i<tempSize ; ++i)
     {
        extraDescriptions.add(new Pair(input.readUTF(), input.readUTF()));
     }
     material = input.readUTF();
     type = input.readInt();
     extraFlags = input.readUTF();
     wearFlags = input.readUTF();
     v0 = input.readUTF();
     v1 = input.readUTF();
     v2 = input.readUTF();
     v3 = input.readUTF();
     v4 = input.readUTF();
     level = input.readByte();
     weight = input.readInt();
     cost = input.readInt();
     cond = input.readUTF();
     immFlags = input.readUTF();
     resFlags = input.readUTF();
     vulnFlags = input.readUTF();
     detectFlags = input.readUTF();
     generalAffects = input.readUTF();

     //I foresee further change of the tab size, so it is managed here
     tempSize = input.readInt();
     for(int i=0 ; i<Math.min(tempSize, localAffects.length) ; ++i)
        localAffects[i] = input.readInt();
     //in case we add some functionnalities
     if (tempSize < localAffects.length)
     {
        for(int i=tempSize ; i<localAffects.length ; ++i)
        {
          localAffects[i] = 0;
        }
     }

     //I foresee further change of the tab size, so it is managed here
     tempSize = input.readInt();
     for(int i=0 ; i<Math.min(tempSize, AnatoliaConsts.objectProgTexts.length) ; ++i)
       specialProgs[i] = input.readUTF();
     //in case we add some functionnalities
     if (tempSize < AnatoliaConsts.mobilePracticerTexts.length)
     {
       for(int i=tempSize ; i<AnatoliaConsts.objectProgTexts.length ; ++i)
         specialProgs[i] = "";
     }

     limit = input.readInt();
  }

  public void writeDescription(FileWriter output) throws IOException
  {
     if ((instances.size() == 0) || isExtern()) return;
     output.write("#"+super.getVnum()+"\n");
     output.write(name+"~"+"\n");
     output.write(shortDescription+"~"+"\n");
     output.write(longDescription+"~"+"\n");
     output.write(material+"~"+"\n");
     output.write(AnatoliaConsts.objectTypeTexts[type].getName().toLowerCase()+" ");
     output.write(extraFlags+" ");
     output.write(wearFlags+" ");
     output.write("\n");
     output.write(v0+" "+v1+" "+v2+" "+v3+" "+v4+"\n");
     output.write(""+level+" "+weight+" "+cost+" "+cond+"\n");

     for(int i=0 ; i<localAffects.length ; ++i)
     {
        if (localAffects[i] != 0)
        {
           output.write("A\n");
           output.write(AnatoliaConsts.applyTexts[i].getCode()+" "+localAffects[i]+"\n");
        }
     }
     char c;

     if (generalAffects.equals("0")) generalAffects = "";
     for(int i=0 ; i<generalAffects.length() ; ++i)
     {
        c = generalAffects.charAt(i);
        output.write("F \n");
        output.write("A 0 0 "+c+"\n");
     }

     if (immFlags.equals("0")) immFlags = "";
     for(int i=0 ; i<immFlags.length() ; ++i)
     {
        c = immFlags.charAt(i);
        output.write("F \n");
        output.write("I 0 0 "+c+"\n");
     }

     if (resFlags.equals("0")) resFlags = "";
     for(int i=0 ; i<resFlags.length() ; ++i)
     {
        c = resFlags.charAt(i);
        output.write("F \n");
        output.write("R 0 0 "+c+"\n");
     }

     if (vulnFlags.equals("0")) vulnFlags = "";
     for(int i=0 ; i<vulnFlags.length() ; ++i)
     {
        c = vulnFlags.charAt(i);
        output.write("F \n");
        output.write("V 0 0 "+c+"\n");
     }
     if (detectFlags.equals("0")) detectFlags = "";
     for(int i=0 ; i<detectFlags.length() ; ++i)
     {
        c = detectFlags.charAt(i);
        output.write("F \n");
        output.write("D 0 0 "+c+"\n");
     }
     Iterator it = extraDescriptions.iterator();
     if (it.hasNext())
     {
       output.write("E"+"\n");
       Pair pair = null;
       while(it.hasNext())
       {
          pair = (Pair) it.next();
          output.write(pair.getFirst()+"~"+"\n");
          output.write(pair.getSecond()+"\n");
          output.write("~"+"\n");
       }
     }
  }

  public void writeReset(FileWriter output) throws IOException
  {
      if (instances.size() == 0) return;
      Iterator it = instances.iterator();
      InstanceOfObject instance = null;
      //look all instance of current object
      AnatoliaRoom anatoliaRoom = null;
      while (it.hasNext())
      {
         instance = (InstanceOfObject) it.next();
         anatoliaRoom = AreaBuilderConsts.getRoomNode(instance.getTheParent());
         //if it's a "normal" instance... (not equiping a mobile)
         if (anatoliaRoom != null)
         {
            output.write("O 0 "+super.getVnum()+" 0 "+anatoliaRoom.getVnum());
            output.write("\t* loads object <"+this.name+"> into room "+anatoliaRoom.roomName);
            output.write("\n");

            //there may be some other object instances in it (if it's a container)
            if (instance.anatoliaObject.type == AnatoliaConsts.OBJECT_TYPE_CONTAINER)
            {
               Map objectsContained = new HashMap(); //<instanceOfObject, nbTimes>
               for(int i=0 ; i<instance.getChildCount() ; ++i)
               {
                  InstanceOfObject object = (InstanceOfObject) instance.getChildAt(i);
                  Integer nb = (Integer) objectsContained.get(object);
                  if (nb == null)
                    nb = new Integer(1);
                  else
                    nb = new Integer(nb.intValue()+1);
                  objectsContained.put(object, nb);
               }

               Set entrySet = objectsContained.entrySet();
               Iterator it2 = entrySet.iterator();
               Map.Entry entry = null;
               while(it2.hasNext())
               {
                 entry = (Map.Entry) it2.next();
                 output.write("P 0 "+((InstanceOfObject) entry.getKey()).anatoliaObject.getVnum()+" 0 "+instance.anatoliaObject.getVnum()+" "+((Integer) entry.getValue()).intValue());
                 output.write("\t* puts obj<"+((InstanceOfObject) entry.getKey()).anatoliaObject.name+"> into obj<"+instance.anatoliaObject.name+">");
                 output.write("\n");
               }
            }
         }
      }
  }

  public void writeProg(FileWriter output) throws IOException
  {
    String s = null;
    for(int i=0 ; i<specialProgs.length ; ++i)
    {
      s = specialProgs[i];
      if ((s != null) && (!s.trim().equals("")))
      {
        output.write("O "+getVnum()+" "+AnatoliaConsts.objectProgTexts[i]+" "+s);
        output.write("\t* object <"+name+">"+"\n");
      }
    }
  }

  public void writeLimit(FileWriter output) throws IOException
  {
    if (limit>0)
    {
      output.write("O "+getVnum()+" "+limit);
      output.write("\t* object <"+name+">"+"\n");
    }
  }

  public void establishLinks()
  {
     LinkedList newInstances = new LinkedList();
     Iterator it = instances.iterator();
     while(it.hasNext())
     {
        newInstances.add(LinkingAndSelectableNodeObject.lookForObjectOfID((Long) it.next()));
     }
     instances = newInstances;
  }
}
