/*********************************************************************
** AreaBuilder is a program designed to build maps for MUDS like    **
** Anatolia.                                                        **
** Copyright (C) 2003 Pierre Chatelier                              **
**                                                                  **
** This program is free software; you can redistribute              **
** it and/or modify it under the terms of the GNU                   **
** General Public License as published by the                       **
** Free Software Foundation, version 2.                             **
**                                                                  **
** This program is distributed in the hope that it will be          **
** useful, but WITHOUT ANY WARRANTY; without even the implied       **
** warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. **
** See the GNU General Public License for more details.             **
**                                                                  **
** You should have received a copy of the                           **
** GNU General Public License along with this program; if not,      **
** write to the Free Software Foundation,                           **
** Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA      **
*********************************************************************/
package areabuilder;

import java.io.*;
import java.util.*; //pour StringTokenizer

public class AnatoliaRoomsReader extends AnatoliaAreReader
{
  public AnatoliaRoomsReader(AreaMap areaMap)
  {
    super(areaMap);
  }

  public void read(BufferedReader input) throws IOException
  {
     String line = super.trim(input);
     while(!line.startsWith("#0"))
     {
       Room room = new Room(areaMap, areaMap.theRooms.anatoliaCollection);
       AnatoliaRoom anatoliaRoom = new AnatoliaRoom(room, areaMap.theRooms.anatoliaCollection);
       room.anatoliaRoom = anatoliaRoom;

       //vnum
       //room.setVnum(MyInteger.parseInt(line.substring(1)));
       anatoliaRoom.imposeVnum(MyInteger.parseInt(line.substring(1)));

       //name
       line = super.trim(input);
       anatoliaRoom.roomName = line.substring(0,Math.max(0,line.length()-1));

       //description
       line = "";
       do
       {
         line = line + "\n" + super.trim(input);
       } while (!line.endsWith("~"));
       anatoliaRoom.roomDescription = line.substring(0,Math.max(0,line.length()-1)).trim();

       //0 flags sectortype
       line = super.trim(input);
       StringTokenizer tokens = new StringTokenizer(line);
       tokens.nextToken(); //one slot has no meaning
       AreaBuilderConsts.stringToSelection(AreaBuilderConsts.interpretFlags(tokens.nextToken()), AnatoliaConsts.roomFlagsTexts, anatoliaRoom.flags);
       anatoliaRoom.sectorType = (byte) Math.max(0,MyInteger.parseInt(tokens.nextToken()));
       anatoliaRoom.sectorType = (byte) Math.min(anatoliaRoom.sectorType,AnatoliaConsts.ROOM_SECTOR_DESERT);

       line = super.trim(input);

       //extras
       while(line.startsWith("E"))
       {
          String keyword = super.trim(input);
          keyword = keyword.substring(0,Math.max(0,keyword.length()-1));
          line = "";
          do
          {
            line = line + "\n" + super.trim(input);
          } while (line.indexOf("~")<0);
          line = line.substring(0,Math.max(0,line.length()-1)).trim();
          String description = new String(line);
          anatoliaRoom.extras.add(new Pair(keyword, description));
          line = super.trim(input);
       }

       //doors
       while(line.startsWith("D"))
       {
         int direction = 0;
         if (line.length()>=2) direction = MyInteger.parseInt(line.substring(1,2));
         anatoliaRoom.doors[direction] = new DoorExit();

         //description
         line = "";
         do
         {
           line = line + "\n" + super.trim(input);
         } while (line.indexOf("~")<0);
         anatoliaRoom.doors[direction].extraDescription = new String(line.substring(1, line.indexOf("~")).trim());

         //door keyword
         line = "";
         do
         {
           line = line + "\n" + super.trim(input);
         } while (line.indexOf("~")<0);
         anatoliaRoom.doors[direction].doorKeyword = line.substring(0,Math.max(0,line.indexOf("~"))).trim();

         //door state key exit_vnum size
         line = super.trim(input);
         tokens = new StringTokenizer(line);
         anatoliaRoom.doors[direction].doorState = (byte) MyInteger.parseInt(tokens.nextToken());
         anatoliaRoom.doors[direction].keyVnum =  MyInteger.parseInt(tokens.nextToken());
         anatoliaRoom.myRoom.connectingToRooms[direction] = new Pair(new Integer(MyInteger.parseInt(tokens.nextToken())), null); //vnum
         anatoliaRoom.doors[direction].doorSize = AnatoliaConsts.SIZE_GARGANTUAN;
         if (tokens.hasMoreTokens())
           anatoliaRoom.doors[direction].doorSize = (byte) MyInteger.parseInt(tokens.nextToken());
         line = super.trim(input);
       }
       //extras
       while(line.startsWith("E"))
       {
          String keyword = super.trim(input);
          keyword = keyword.substring(0,Math.max(0,keyword.length()-1));
          line = "";
          do
          {
            line = line + "\n" + super.trim(input);
          } while (line.indexOf("~")<0);
          String description = new String(line.substring(0,Math.max(0,line.indexOf("~")))).trim();
          anatoliaRoom.extras.add(new Pair(keyword, description));
          if (!line.endsWith("S")) line = super.trim(input);
       }

       //MP & HP recovery rates
       while (line.startsWith("M") || line.startsWith("H"))
       {
         if (line.startsWith("M"))
         {
           tokens = new StringTokenizer(line);
           tokens.nextToken(); //reads M
           anatoliaRoom.MPRecovery = MyInteger.parseInt(tokens.nextToken());
           if (tokens.hasMoreTokens())
           {
             tokens.nextToken(); //reads H
             anatoliaRoom.HPRecovery = MyInteger.parseInt(tokens.nextToken());
           }
         }
         if (line.startsWith("H"))
         {
           tokens = new StringTokenizer(line);
           tokens.nextToken(); //reads H
           anatoliaRoom.HPRecovery = MyInteger.parseInt(tokens.nextToken());
           if (tokens.hasMoreTokens())
           {
             tokens.nextToken(); //reads M
             anatoliaRoom.MPRecovery = MyInteger.parseInt(tokens.nextToken());
           }
         }
         line = super.trim(input);
       }//end while M & H

       //expecting the end ("S")
       while(!line.endsWith("S")) line = super.trim(input);
       line = super.trim(input);
       areaMap.theRooms.add(room);
    }//end while room
  } //end function
} //end class