/*********************************************************************
** AreaBuilder is a program designed to build maps for MUDS like    **
** Anatolia.                                                        **
** Copyright (C) 2003 Pierre Chatelier                              **
**                                                                  **
** This program is free software; you can redistribute              **
** it and/or modify it under the terms of the GNU                   **
** General Public License as published by the                       **
** Free Software Foundation, version 2.                             **
**                                                                  **
** This program is distributed in the hope that it will be          **
** useful, but WITHOUT ANY WARRANTY; without even the implied       **
** warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. **
** See the GNU General Public License for more details.             **
**                                                                  **
** You should have received a copy of the                           **
** GNU General Public License along with this program; if not,      **
** write to the Free Software Foundation,                           **
** Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA      **
*********************************************************************/
package areabuilder;

import java.util.*;

public class UndoableActionAreaMapGroupRoomClone extends UndoableActionAreaMapGroupRoom
{
   public UndoableActionAreaMapGroupRoomClone(AreaMap areaMap, SelectionOfRooms selection, boolean automatic)
   {
      super(areaMap, selection, automatic);
   }
   public UndoableActionAreaMapGroupRoomClone(AreaMap areaMap, SelectionOfRooms selection)
   {
      super(areaMap, selection);
   }

   public void doAction()
   {
      Map mapOfClonedRooms = new HashMap(); //<room, clonedRooms>

      Room room = null;
      UndoableActionAreaMapRoomClone action = null;
      Iterator it = copyOfSelection.getIterator();
      Room roomDragged = areaMap.roomDragged; //save before being modified by sub-action of Room cloning
      while (it.hasNext())
      {
         room = (Room) it.next();
         action = new UndoableActionAreaMapRoomClone(areaMap, room, UndoableAction.AUTOMATIC_ACTION);
         this.actionStack.doAction(action);
         mapOfClonedRooms.put(room, action.getClonedRoom());
      }

      //now we add the intern wires;
      it = copyOfSelection.getListOfInternWiresIterator();
      Wire wire = null;
      while (it.hasNext())
      {
         wire = (Wire) ((Wire) it.next()).clone();
         wire.reconnect((Room) mapOfClonedRooms.get(wire.getFromRoom()), wire.getFromDoor(),
                        (Room) mapOfClonedRooms.get(wire.getToRoom()), wire.getToDoor());
         this.actionStack.doAction(
            new UndoableActionAreaMapWireAdd(areaMap, wire, UndoableAction.AUTOMATIC_ACTION)
        );
      }

      //then we rebuild selection
      areaMap.selectedRooms.clear();
      Set entries = mapOfClonedRooms.entrySet();
      it = entries.iterator();
      while (it.hasNext())
      {
         areaMap.selectedRooms.addToSelection((Room) ((Map.Entry) it.next()).getValue());
      }
      areaMap.roomDragged = (Room) mapOfClonedRooms.get(roomDragged);
   }

   public void undoAction()
   {
       this.actionStack.undoAllAction();

       //swap selections of rooms
       SelectionOfRooms tempSelected = (SelectionOfRooms) areaMap.selectedRooms.clone();
       areaMap.selectedRooms.clear();
       areaMap.selectedRooms = (SelectionOfRooms) copyOfSelection.clone();
       copyOfSelection = tempSelected;
   }

   public void redoAction()
   {
       this.actionStack.redoAllAction();

       SelectionOfRooms tempSelected = (SelectionOfRooms) areaMap.selectedRooms.clone();
       areaMap.selectedRooms.clear();
       areaMap.selectedRooms = (SelectionOfRooms) copyOfSelection.clone();
       copyOfSelection = tempSelected;
   }
}