/*********************************************************************
** AreaBuilder is a program designed to build maps for MUDS like    **
** Anatolia.                                                        **
** Copyright (C) 2003 Pierre Chatelier                              **
**                                                                  **
** This program is free software; you can redistribute              **
** it and/or modify it under the terms of the GNU                   **
** General Public License as published by the                       **
** Free Software Foundation, version 2.                             **
**                                                                  **
** This program is distributed in the hope that it will be          **
** useful, but WITHOUT ANY WARRANTY; without even the implied       **
** warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. **
** See the GNU General Public License for more details.             **
**                                                                  **
** You should have received a copy of the                           **
** GNU General Public License along with this program; if not,      **
** write to the Free Software Foundation,                           **
** Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA      **
*********************************************************************/
package areabuilder;

import java.util.*; //for Iterator

public class UndoableActionAreaMapRoomRemove extends UndoableActionAreaMapRoom
{
  public UndoableActionAreaMapRoomRemove(AreaMap areaMap, Room room, boolean automatic)
  {
    super(areaMap, room, automatic);
  }
  public UndoableActionAreaMapRoomRemove(AreaMap areaMap, Room room)
  {
    super(areaMap, room);
  }

  public void doAction()
  {
    Pair[] connectingToRooms = room.getConnectingToRooms();
    Pair pair = null;
    for(byte door=1; door <=6; ++door)
    {
       pair = connectingToRooms[door-1];
       if (pair != null)
          this.actionStack.doAction(
             new UndoableActionAreaMapWireRemove(areaMap, (Wire) pair.getSecond(), UndoableAction.AUTOMATIC_ACTION)
          );
    }
    Iterator it = room.getMayComeFromRooms().listIterator();
    Room fromRoom = null;
    byte fromDoor = 0;
    Wire connectingWire = null;
    while (it.hasNext())
    {
       pair = (Pair) it.next();
       fromRoom = (Room) pair.getFirst();
       fromDoor = ((Byte) pair.getSecond()).byteValue();
       pair = (Pair) fromRoom.getConnectingToRooms()[fromDoor];
       if (pair != null)
       {
          connectingWire = (Wire) pair.getSecond();
          this.actionStack.doAction(
             new UndoableActionAreaMapWireRemove(areaMap,
                                              connectingWire, UndoableAction.AUTOMATIC_ACTION)
           );
       }
    }
    areaMap.theRooms.remove(room);
    areaMap.objectTree.delTree(room.anatoliaRoom, actionStack);
  }

  public void undoAction()
  {
     areaMap.theRooms.add(room);
     this.actionStack.undoAllAction();
  }

  public void redoAction()
  {
     actionStack.clear();
     doAction();
  }

}